/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"

Board::Board(int size) {
	this->size = size;
	this->width = size*4 - 1;
	this->height = size*2;
	this->board = (u8*)malloc(sizeof(u8)*width*height);

	for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++)
			this->board[x+y*width] = initValue(x, y)-1;
}

int
Board::getSize() const {
	return size;
}

int
Board::getWidth() const {
	return width;
}

int
Board::getHeight() const {
	return height;
}

int
Board::initValue(int x, int y) const {
	if (pointsDown(x, y)) {
		// top border
	 	if (!isValidCell(x, y-1))
			return 3;
	}
	// bottom border
	else if (!isValidCell(x, y+1))
		return 3;
	// left or right border
	if (!isValidCell(x-1, y) || !isValidCell(x+1, y))
		return 3;
	// other
	return 4;
}

bool
Board::pointsDown(const TCoordCell& c) const {
	return pointsDown(c.col, c.row);
}

bool
Board::pointsDown(int x, int y) const {
	return ((y % 2 == 0) == (x % 2 == 0)) == (size % 2 == 0);
}

bool
Board::isValidCell(const TCoordCell& c) const {
	return isValidCell(c.col, c.row);
}

bool
Board::isValidCell(int x, int y) const {
	if (x < 0 || y < 0 || x >= width || y >= height)
		return false;
	if (x + y < size-1)
		return false;
	if ((width-x-1) + (height-y-1) < size-1)
		return false;
	if ((width-x-1) + y < size-1)
		return false;
	if ((height-y-1) + x < size-1)
		return false;
	return true;
}

bool
Board::checkEnd(TPlayer player) const {
	for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++) {
			if (isValidCell(x, y))
				if (getCellPlayer(x, y) != player)
					return false;
		}
	return true;
}

bool
Board::checkLost(const Player& player) const {
	for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++) {
			if (isValidCell(x, y))
				if (getCellPlayer(x, y) == player.getType()
						|| (player.hasStarter() && getCellPlayer(x, y) == 0))
					return false;
		}
	return true;
}

u8
Board::getCellValue(TCoordCell cell) const {
	return getCellValue(cell.col, cell.row);
}

u8
Board::getCellValue(int x, int y) const {
	return board[x+y*width] & 15;
}

TPlayer
Board::getCellPlayer(TCoordCell cell) const {
	return getCellPlayer(cell.col, cell.row);
}

TPlayer
Board::getCellPlayer(int x, int y) const {
	return (TPlayer) (board[x+y*width] >> 5);
}

void
Board::setCellValue(TCoordCell cell, u8 value) {
	setCellValue(cell.col, cell.row, value);
}

void
Board::setCellValue(int x, int y, u8 value) {
	board[x+y*width] = value | (getCellPlayer(x, y)<<5);
}

void
Board::setCellPlayer(TCoordCell cell, TPlayer player) {
	setCellPlayer(cell.col, cell.row, player);
}

void
Board::setCellPlayer(int x, int y, TPlayer player) {
	board[x+y*width] = (player<<5) | getCellValue(x, y);

}

