/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "game.h"

Game::Game(UI* ui, Options* options) {
	int starters = -1;
	if (options->rules == TACTICAL_OPTION) {
		switch (options->boardSize) {
			case SMALL_OPTION: starters = 1; break;
			case MEDIUM_OPTION: starters = 2; break;
			case BIG_OPTION: starters = 3; break;
		}
	}
	players[0] = new Player(BLUE_PLAYER, starters);
	players[1] = new Player(RED_PLAYER, starters);
	players[2] = new Player(GREEN_PLAYER, starters);

	board = new Board(options->boardSize);
	ai = 0;
	if (options->aiPlayer != NONE_PLAYER)
		ai = new AIEasy(options->aiPlayer, *board);

	this->ui = ui;
	this->options = options;
	buffer = new CellBuffer();

	skip = -1;
	curPlayer = 0;
	ui->drawBoard(*this->board);
	ui->showCurrentPlayer(getCurrentPlayer()->getType(), getCurrentPlayer()->startersLeft(), *board);
	state = PLAY_STATE;
}

Game::~Game() {
	if (state == PAUSE_STATE)
		toggleHelp();
	delete board;
	delete ai;
	delete buffer;
}

const Board&
Game::getBoard() {
	return *this->board;
}

Player *
Game::getCurrentPlayer() {
	if (curPlayer == 3)
		return players[1];
	return players[curPlayer];
}

void
Game::nextPlayer() {
	curPlayer = (curPlayer+1) % options->numPlayers;
	if (skip == getCurrentPlayer()->getType())
		nextPlayer();
}

void
Game::onQuarterSecond() {
}

void
Game::turn(const TCoordCell& cell) {
	// abort if game is not running
  if (state != PLAY_STATE)
    return;
	// abort if not a valid cell
	if (!board->isValidCell(cell))
		return;
	
	TPlayer cellPlayer = board->getCellPlayer(cell);

	if (cellPlayer == NONE_PLAYER) {
		// abort if cell is vacant and no starters left
		if (!getCurrentPlayer()->hasStarter())
			return;
		getCurrentPlayer()->useStarter();
	}
	else {
		// abort if cell already occupied
		if (cellPlayer != getCurrentPlayer()->getType())
			return;
	}

	setCell(cell, getCurrentPlayer()->getType());
	while (!buffer->isEmpty())
		resetCell(buffer->remove(), getCurrentPlayer()->getType());

	// end if current player won
	if (board->checkEnd(getCurrentPlayer()->getType())) {
		state = END_STATE;
		ui->showWinner(getCurrentPlayer()->getType());
		return;
	}

	// in 2 vs 1 mode, end if one of the 2 lost
	if (options->numPlayers == 4
			&& (board->checkLost(*players[0]) || board->checkLost(*players[2]))) {
		state = END_STATE;
		ui->showWinner(RED_PLAYER);
		return;
	}

	nextPlayer();
	if (options->numPlayers == 2) {
		if (board->checkLost(*getCurrentPlayer())) {			
			state = END_STATE;
			nextPlayer();
			ui->showWinner(getCurrentPlayer()->getType());
			return;
		}
	}
	else if (options->numPlayers == 3) {
		if (board->checkLost(*getCurrentPlayer())) {
			// first player lost
			if (skip == -1) {
				skip = getCurrentPlayer()->getType();
				nextPlayer();
			}
			// second player lost
			else {
				state = END_STATE;
				nextPlayer();
				ui->showWinner(getCurrentPlayer()->getType());
				return;
			}
		}
	}
	ui->showCurrentPlayer(getCurrentPlayer()->getType(), getCurrentPlayer()->startersLeft(), *board);
}

void Game::startAI() {
	if (state != PLAY_STATE)
		return;
	TCoordCell step = ai->step(getCurrentPlayer()->startersLeft());
	for (int i = 0; i < 20; i++)
		swiWaitForVBlank();
	turn(step);
}

inline void
Game::addTask(const TCoordCell& cell) {
	if (buffer->isFull())
		resetCell(buffer->remove(), getCurrentPlayer()->getType());
	else
		this->buffer->add(cell);
}

bool
Game::humansTurn() {
	return (getCurrentPlayer()->getType() != options->aiPlayer);
}

void
Game::setCell(TCoordCell cell, TPlayer player) {
	if (!board->isValidCell(cell))
		return;
	// abort, if cell already queued 
	if (board->getCellValue(cell) == 0)
		return;

	// decrease value by one
	board->setCellValue(cell, board->getCellValue(cell)-1);
	// set player to current
	board->setCellPlayer(cell, player);
	ui->drawCell(cell, *board);

	if (board->getCellValue(cell) == 0)
		addTask(cell);
}

void
Game::resetCell(TCoordCell cell, TPlayer player) {
	// a small timeout
	for (int i = 0; i < 18; i++)
		swiWaitForVBlank();

	// reset to initial value
	board->setCellValue(cell, board->initValue(cell.col, cell.row));
	ui->drawCell(cell, *board);

	// start chain reaction
	setCell(cell+TCoordCell(0, board->pointsDown(cell) ? -1 : 1), player);
	setCell(cell+TCoordCell(-1, 0), player);
	setCell(cell+TCoordCell(1, 0), player);
}

void
Game::toggleHelp() {
	switch (state) {
		case PLAY_STATE: state = PAUSE_STATE; break;
		case PAUSE_STATE: state = PLAY_STATE; break;
		default: break;
	} 
	ui->toggleHelp(*board);
}
