/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "options_ui.h"

#include "top.h"

OptionsUI::OptionsUI(Options options, Audio *audio) {
	this->options = options;
	this->audio = audio;
	this->selected = 0;
	this->changed = false;

	// init sub screen
	videoSetModeSub(MODE_3_2D);

	// set up text background
	this->console = *consoleInit(NULL, 0, BgType_Text4bpp, BgSize_T_256x256, 2, 0, false, true);

	// set up bitmap background
	int id = bgInitSub(3, BgType_Bmp8, BgSize_B8_256x256, 2, 0);
	dmaCopy(topBitmap, bgGetGfxPtr(id), topBitmapLen);
	dmaCopy(topPal, BG_PALETTE_SUB, topPalLen);
	BG_PALETTE_SUB[255] = RGB15(31, 31, 31);

	consoleClear();
	iprintf("\x1b[10;3HBoard:\x1b[11;3HMode:\x1b[12;3HRules:\x1b[13;3HAI:\x1b[14;3HSound:");
	printCurrent();
	iprintf("\x1b[17;1HWelcome to jumpDS!");
	iprintf("\x1b[19;1HPress SELECT");
	iprintf("\x1b[20;1Hfor help.");
}

void
OptionsUI::printCurrent() {
	consoleSelect(&console);
	for (int i = 0; i < 5; i++)
		iprintf("\x1b[1%d;2H%s", i, selected==i ? "\7" : " ");

	// board size
	switch (options.boardSize) {
		case SMALL_OPTION: iprintf("\x1b[10;10Hsmall  "); break;
		case MEDIUM_OPTION: iprintf("\x1b[10;10Hmedium"); break;
		case BIG_OPTION: iprintf("\x1b[10;10Hbig      "); break;
	}

	// number of players / mode
	iprintf("\x1b[11;10H");
	if (options.numPlayers == 4)
		iprintf("2 vs 1   ");
	else
		iprintf("%d players", options.numPlayers);

	// ruleset
	switch (options.rules) {
		case STANDARD_OPTION: iprintf("\x1b[12;10Hstandard"); break;
		case TACTICAL_OPTION: iprintf("\x1b[12;10Htactical"); break;
	}

	// AI player
	iprintf("\x1b[13;10H%s  ", TPLAYER_STRING[options.aiPlayer]);

	switch (options.sound) {
		case ALL_OPTION:     iprintf("\x1b[14;10Hall    "); break;
		case MUSIC_OPTION:   iprintf("\x1b[14;10Hmusic  "); break;
		case EFFECTS_OPTION: iprintf("\x1b[14;10Heffects"); break;
	}

	if (this->changed) {
		iprintf("\x1b[16;1HSettings have been\x1b[17;1Hchanged and will ");
		iprintf("\x1b[18;1Htake effect after\x1b[19;1Hthe next restart.");
	} else {
		iprintf("\x1b[16;1H                  \x1b[17;1H                  ");
		iprintf("\x1b[18;1H                 \x1b[19;1H                 ");
	}
	iprintf("\x1b[20;1H                 ");
}

void
OptionsUI::navigate(int key) {
	if (key & KEY_UP)
		selected = selected == 0 ? 4 : (selected-1);
	else if (key & KEY_DOWN)
		selected = (selected+1) % 5;
	else {
		switch (selected) {
			case 0:
				switch (options.boardSize) {
					case SMALL_OPTION: options.boardSize = key & KEY_RIGHT ? MEDIUM_OPTION : BIG_OPTION; break;
					case MEDIUM_OPTION: options.boardSize = key & KEY_RIGHT ? BIG_OPTION : SMALL_OPTION; break;
					case BIG_OPTION: options.boardSize = key & KEY_RIGHT ? SMALL_OPTION : MEDIUM_OPTION; break;
				}
				changed = true;
				break;
			case 1:
				switch (options.numPlayers) {
					case 2: options.numPlayers = key & KEY_RIGHT ? 3 : 4; break;
					case 3: options.numPlayers = key & KEY_RIGHT ? 4 : 2; break;
					case 4: options.numPlayers = key & KEY_RIGHT ? 2 : 3; break;
				}
				if (options.numPlayers == 2 && options.aiPlayer == GREEN_PLAYER)
					options.aiPlayer = NONE_PLAYER;
				changed = true;
				break;
			case 2:
				switch (options.rules) {
					case STANDARD_OPTION: options.rules = TACTICAL_OPTION; break;
					case TACTICAL_OPTION: options.rules = STANDARD_OPTION; break;
				}
				changed = true;
				break;
			case 3:
				{
					int max = options.numPlayers == 4 ? 4 : options.numPlayers+1;
					options.aiPlayer = (TPlayer) ((options.aiPlayer + (key & KEY_RIGHT ? 1 : -1) + max) % max);
					changed = true;
				}
				break;
			case 4:
				switch (options.sound) {
					case ALL_OPTION:
						options.sound = (key & KEY_RIGHT) ? MUSIC_OPTION : EFFECTS_OPTION;
						break;
					case MUSIC_OPTION:
						options.sound = (key & KEY_RIGHT) ? EFFECTS_OPTION : ALL_OPTION;
						break;
					case EFFECTS_OPTION:
						options.sound = key & KEY_RIGHT ? ALL_OPTION : MUSIC_OPTION;
						break;
				}
				// update audio directly
				audio->update(this->options.sound);
				break;
		}
	}
	printCurrent();
}

void
OptionsUI::update(Options *options) {
	changed = false;
	printCurrent();
	*options = this->options;
}
